/**************************** NN CONFIGURATION ***************************
 *
 *	Configuration file for nn
 *
 *	You must edit this file to reflect your local configuration
 *	and environment.
 *
 *	Follow the instructions given in the comments.  See the files
 *	INSTALLATION, NNTP, and PROBLEMS for more details.
 */

#define	RELEASE 	6
#define	VERSION 	3


#include <stdio.h>
#include <ctype.h>


/*********************** NETWORK DEPENDENT DEFINITIONS **********************
 *
 *	Define NETWORK_DATABASE if you share the database through NFS on 
 *	a network with different, non-compatible machines, e.g. SUNs and
 *	VAXen, or SUN-3 and SUN-4, or if you are using different compilers
 *	on the same architecture.
 *
 *	In a homogenous network, you can leave it undefined for higher
 *	performance (no data conversion is needed).
 */

#define NETWORK_DATABASE	/* */


/********************************** NNTP *********************************
 *
 * 	Define NNTP to enable nntp support.
 *
 *	With NNTP, the nnmaster still maintains a local database of 
 *	all article headers for fast access (and because NNTP does not 
 *	support nn - yet), while the articles are fetched from the
 *	nntp server when they are read or saved.
 *
 *	You may still share this database through NFS locally (see the 
 *	description of NETWORK_DATABASE above) if you don't want to
 *	have separate nn databases on all your local systems.
 *
 *	Consult the file NNTP for further information on the use of NNTP.
 */

/* #define NNTP			/* */

/* 
 * 	Define NNTP_SERVER to the name of a file containing the name of the
 * 	nntp server.
 *
 * 	It is vital that both the nnmaster and all nn users on a machine 
 *	uses the same nntp server, because the nn database is synchronized
 *	with a specific news active file. 
 *
 *	If the file name does not start with a slash, it is relative to 
 *	LIB_DIRECTORY defined below.
 */

#define NNTP_SERVER	"/etc/newsserver"

/*
 *    Define NNTP_POST if you want nn to reject attempts to post via
 *    NNTP to a server, that disallows postings.
 *
 *    You should define this, if you use the NNTP based inews for
 *    postings from NNTP clients.  If you use another mechanism, that
 *    does not involve NNTP, you should leave it undefined.
 */

#define NNTP_POST             /* */


/***************** OPERATING SYSTEM DEPENDENT DEFINITIONS *******************
 *
 *  	Include the appropriate s- file for your system below.
 *
 *	If a file does not exist for your system, you can use s-template.h
 *	as a starting point for writing you own.
 */

#include "s-sunos4-0.h"


/********************** MACHINE DEPENDENT DEFINITIONS **********************
 *
 *	Include the appropriate m- file for your system below.
 *	
 *	If a file does not exist for your system, you can use m-template.h
 *	as a starting point for writing you own.
 */

#include "m-sparc.h"


/******************** SITE DEPENDENT DEFINITIONS **********************
 *
 *	Edit the following part to suit your local system setup
 */

/*
 *	Specify where programs and data should be placed
 *
 *	BIN_DIRECTORY   - the location of the user programs
 *	LIB_DIRECTORY	- the location of auxiliary programs and files
 *	DB_DIRECTORY	- the directory containing the nn database
 *
 *
 * notice: if you share the news directory accross a network, you should
 *	   use something like /usr/spool/news/.nn for DB_DIRECTORY.
 */

#define BIN_DIRECTORY	"/usr/news/nn/bin"
#define LIB_DIRECTORY	"/usr/news/nn/lib"
#define DB_DIRECTORY	"/usr/news/nn/db"

/*
 * 	Specify directories for the user and system manuals
 *
 * 	Adapt this to your local standards; the manuals will be named 
 * 		$(MAN_DIR)/program.$(MAN_SECTION)
 */

#define USER_MAN_DIR 	"/usr/man/man1"
#define USER_MAN_SECTION 	"1"

#define SYS_MAN_DIR 	"/usr/man/man1"
#define SYS_MAN_SECTION 	"1m"

/*
 *	Specify where to put temporary files.  Overriden by $TMPDIR.
 *	Notice that nn does not create "large" temp files.
 */

#define TMP_DIRECTORY	"/tmp"

/*
 * Specify owner and group for files belonging to this package.
 *
 * Specifically, the nnmaster will run suid/sgid to this owner and group.
 *
 * The only requirements are that the ownership allows the nnmaster to
 * READ the news related files and directories, and the ordinary users
 * to read the database and execute the nn* programs.
 *
 * Normal choices are:  (news, news)  and   (your uid, your gid)
 */

#define OWNER	"news"
#define	GROUP	"news"

/*
 *	Define STATISTICS if you want to keep a record of how much
 *	time the users spend on news reading.
 *
 *	Sessions shorter than the specified number of minutes are not
 *	recorded (don't clutter up the log file).
 *
 *	This is entered into the file $LIB_DIRECTORY/Log with code U
 */

/* #define STATISTICS	5 /* minutes */

/*
 *	Define HAVE_ROUTING if your mailer understands domain based
 *	adresses (...@...) and performs the necessary rerouting (e.g. 
 *	Sendmail or Smail).
 *
 *	Otherwise, nn will provide a simple routing facility using
 *      routing information specified in the file LIB_DIRECTORY/routes.
 */

#define HAVE_ROUTING			/* */

/*
 *	If HAVE_ROUTING is NOT defined, nn needs to know the name of
 * 	your host.  To obtain the host name it will use either of the
 *	'uname' or 'gethostname' system calls as specified in the s- file 
 *	included above.
 *
 *	If neither 'uname' nor 'gethostname' is available, you must
 *	define HOSTNAME to be the name of your host.  Otherwise, leave
 *	it undefined (it will not be used anyway).
 */

/* #define HOSTNAME	"myhost"	/* Not used if HAVE_ROUTING */

/*
 *	Specify the location of your news programs and files
 */

#define	INEWS_PATH	"/usr/news/lib"
#define NEWS_ACTIVE	"/usr/news/lib/active"
#define NEWS_DIRECTORY	"/usr/news/spool"

/*
 *	Specify a mailer that accepts a letter WITH a header IN THE TEXT.
 *
 * 	A program named 'recmail' program is normally delivered with 
 *	the news system.
 *	On BSD systems you can also use "/usr/lib/sendmail -t".
 */

#define REC_MAIL	"/usr/lib/news/recmail"

/*
 *	Define APPEND_SIGNATURE if you want nn to ask users to append
 *	~/.signature to mail messages (reply/forward/mail).  
 *	
 *	If the mailer defined in REC_MAIL automatically includes .signature
 *	you should not define this (it will fool people to include it twice).
 *
 *	I think 'recmail' includes .signature, but 'sendmail -t' doesn't.
 */

/* #define APPEND_SIGNATURE		/* */

/*
 *	Default folder directory
 */

#define FOLDER_DIRECTORY	"~/News"

/*
 *	Max length of authors name (in "edited" format).
 *	Also size of "Name" field on the article menus.
 *	You may want to increase this if your terminals are wider than
 *	80 columns.
 */

#define NAME_LENGTH 		16

/*
 * 	Define RESIZING to make nn understand dynamic window-resizing.
 * 	(It uses the TIOCGWINSZ ioctl found on most 4.3BSD systems)
 */

/* #define RESIZING		/* */


/*************************** GOPHER STUFF ********************************/
/*
 *	Where to put the hierarchy of links.
 */
#define GOPHER_NEWS_DIR	"/usr/news/gopher/data"
/*
 *	The Host= and Port= entries to put in the link files.
 */
#define GOPHER_HOST	"msuinfo.msu.edu"
#define GOPHER_PORT	"71"

/************************ CONFIGURATION COMPLETED ************************
 *
 *	The rest of this file will not need any changes.
 */

#include "global.h"
