/*
 *	Gnews - USENET to Gopher gateway.
 *
 *	Written by: Ed Symanzik on 2/4/92
 *	Written for: Michigan State University
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <errno.h>
#include "config.h"
#include "db.h"

export int Debug = 0;
static int verbose = 1;

extern int errno;

export char gopher_host[] = GOPHER_HOST;
export char gopher_port[] = GOPHER_PORT;
export char gopher_news_dir[] = GOPHER_NEWS_DIR;

main(argc, argv)
int argc;
char *argv[];
{
	int cur_group = -1;
	register group_header *gh;
	char dir_name[FILENAME];
	char art_dir[FILENAME];
	char *p, *q, *r;

	open_master(OPEN_READ);
	visit_active_file();

	strcpy(dir_name, gopher_news_dir);
	p = dir_name + strlen(dir_name);
	*p++ = '/';
	while (++cur_group < master.number_of_groups) {
		gh = &active_groups[cur_group];
		for (q = p, r = gh->group_name; *r; r++) {
			if (*r == '.') {
				*q = '\0';
				make_group_dir(dir_name);
				*q++ = '/';
			} else {
				*q++ = *r;
			}
		}
		*q = '\0';
		make_group_dir(dir_name);
		strcpy(art_dir, p);
		strcat(dir_name, "/.link");
		make_link_file(gh, dir_name, art_dir);
	} 
    nn_exit(0);
}

make_group_dir(dir_name)
char *dir_name;
{
	if (mkdir(dir_name, 0777) == -1) {
		if (errno == EEXIST) {
			if (Debug) 
				printf("exists: %s\n",dir_name);
		} else {
			if (Debug) 
				perror(dir_name);
		}
	} else {
		if (Debug) 
			printf("created: %s\n", dir_name);
	}
}


/*
 * This part is mostly nn code.  If admin:dump_group() changes in future
 * versions of nn, this may have to be updated.
 */
static make_link_file(gh, link_name, art_dir)
group_header *gh;
char *link_name;
char *art_dir;
{
    FILE			*data, *ix, *link;
    data_header			hdr;
    off_t	  		data_offset, real_offset, next_offset;
    cross_post_number		cross_post;
    article_number		first_article, next_article, this_art;
    int				n, was_digest;
    char			buffer[512];

    if (init_group(gh) <= 0)
        printf("cannot access group %s\n", gh->group_name);

    update_group(gh);
    
    first_article = gh->first_l_article;
    if (first_article <= 0) first_article = 1;
    
    ix = open_data_file(gh, 'x', OPEN_READ);
    if (ix == NULL) {
	if (verbose) printf("NO INDEX FILE\n");
	return 0;
    }
    next_offset = get_index_offset(gh, first_article);
    fseek(ix, next_offset, 0);
    
    if (!db_read_offset(ix, &real_offset)) {
	if (verbose) printf("NO INDEX FOR ARTICLE %ld\n", (long)first_article);
	fclose(ix);
	return 0;
    }

    data_offset = real_offset;
    fseek(ix, next_offset, 0);
    
    data = open_data_file(gh, 'd', OPEN_READ);
    if (data == NULL) {
	if (verbose) printf("NO DATA FILE\n");
	fclose(ix);
	return 0;
    }

	if ((link = fopen(link_name, "w")) == NULL) {
		if (verbose) printf("CANNOT OPEN LINK FILE\n");
		fclose(data);
		fclose(ix);
		return 0;
	}
    
    next_article = first_article;
    was_digest = 0;
    
    while (next_article <= gh->last_l_article) {
	if (!db_read_offset(ix, &real_offset)) {
	    if (verbose)
		printf("NO INDEX FOR ARTICLE %ld\n", (long)next_article);
	    goto err;
	}

	if (data_offset != real_offset)
	    goto ix_data_err;
	
	fseek(data, data_offset, 0);

     in_digest:

	next_offset = data_offset;
	if (!db_read_art(data, &hdr, &data_offset)) {
	    if (real_offset == gh->data_write_offset || was_digest)
		break;
	    
	    if (verbose) printf("No article header for article # %ld\n",
				(long)next_article);
	    goto err;
	}
	
	if (hdr.dh_number) {
	    if (was_digest) {
		next_article++;
		if (!db_read_offset(ix, &real_offset)) 
		    goto ix_eof_err;
		if (real_offset != next_offset)
		    goto ix_data_err;
	    }
	    
	    this_art = hdr.dh_number < 0 ? -hdr.dh_number : hdr.dh_number;
	
	    if (this_art != next_article) {
		if (this_art < next_article) {
		    if (verbose)
			printf("Article # %ld out of sequence\n", 
			       (long)this_art);
		    goto err;
		}
/*		
		if (this_art == (next_article + 1)) 
		    printf("Article # %ld missing (OK)\n",
			   (long)next_article);
		else
		    printf("Articles # %ld -> %ld missing (OK)\n",
			   (long)next_article, (long)this_art);
*/		
		while (this_art > next_article) {
		    if (!db_read_offset(ix, &next_offset))
			goto ix_eof_err;
		    if (next_offset != real_offset) 
			goto ix_data_err;
		    next_article++;
		}
	    }
	}

	if (hdr.dh_lpos == (off_t)0) {	/* article not accessible */
	    if (verbose)
		printf("# %ld: NO ARTICLE (ok)\n", (long)next_article);
	    continue;
	}
    
	fprintf(link, "#\nType=0\n");
	fprintf(link, "Port=%s\n", gopher_port);
	fprintf(link, "Host=%s\n", gopher_host);
	/* gopher does a chroot */
	fprintf(link,"Path=/%s/%d\n", art_dir, hdr.dh_number);

	data_offset += hdr.dh_cross_postings * sizeof(cross_post_number)
	    + hdr.dh_sender_length 
	    + hdr.dh_subject_length;

	if (hdr.dh_cross_postings) {
	
	    n = hdr.dh_cross_postings;
	    while (--n >= 0) {
		if (fread((char *)&cross_post, sizeof(cross_post_number), 1, data)!= 1)
		    goto data_error;
#ifdef NETWORK_DATABASE
#ifndef NETWORK_BYTE_ORDER
		cross_post = ntohl(cross_post);
#endif
#endif		
	    }
	}	


	if (hdr.dh_sender_length) {
	    if (fread(buffer, sizeof(char), (int)hdr.dh_sender_length, data)
		!= hdr.dh_sender_length) goto data_error;
	    buffer[hdr.dh_sender_length] = NUL;
	}
	
	if (hdr.dh_subject_length) {
	    if (fread(buffer, sizeof(char), (int)hdr.dh_subject_length, data)
		!=  hdr.dh_subject_length) goto data_error;
	    buffer[(hdr.dh_subject_length>70) ? 70:hdr.dh_subject_length] = NUL;
            fprintf(link, "Name=%s\n", buffer);
	}

	if (IS_DIGEST_HEADER(hdr) || IS_SUB_DIGEST(hdr)) {
	    was_digest = 1;
	    goto in_digest;
	}
	    
	was_digest = 0;
	next_article++;

    }

out:

    if (data_offset != gh->data_write_offset) {
	if (verbose)
	    printf("\n*** ERROR: Next data offset %ld wrong. real offset=%ld\n",
		   gh->data_write_offset, data_offset);
	goto err;
    }
    
    fclose(link);
    fclose(data);
    fclose(ix);
    return 1;

    
    
 data_error:
    if (verbose) printf("\n*** END OF FILE on DATA FILE\n\n");
    goto err;

 ix_eof_err:
    if (verbose) printf("NO INDEX FOR ARTICLE %ld\n", (long)next_article);
    goto err;
    
 ix_data_err:
    if (verbose) {
	printf("\n*** OFFSET ERROR in article # %ld offsets:\n",
	       (long)next_article);
	printf("    Calcuated offset %ld differs from index %ld\n",
	       (long)data_offset, (long)real_offset);
    }
    goto err;
    
 err:
	    	
    fclose(link);
    fclose(data);
    fclose(ix);
    return 0;
}

/* 
 * nn_exit() --- called whenever a program exits.
 */
 
nn_exit(n)
int n;
{
    static int loop = 0;
    
    if (loop) exit(n);
    loop++;

#ifdef NNTP
    nntp_cleanup();
#endif /* NNTP */
    close_master();
    
    exit(n);
}

/* dummy routines to stop make from complaning */

group_header *add_new_group(name)
char *name;
{
	return NULL;
}

int user_error(msg)
char *msg;
{
	return 0;
}

#ifdef HAVE_JOBCONTROL
suspend_nn()
{}
#endif
